package com.pitong.business;


import com.pitong.business.item.DataPackage;
import com.pitong.business.item.Picture;
import com.pitong.business.service.picPro.picImpl.*;
import com.pitong.business.service.transfer.FindService;
import com.pitong.business.service.transfer.TransferService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.*;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;


@Service
@SpringBootApplication
public class Application implements ApplicationRunner {

    @Autowired
    private FindService findService;
    @Autowired
    private TransferService transferService;

    public static void main(String[] args) {
        new SpringApplicationBuilder(Application.class)
                .web(WebApplicationType.NONE) // .REACTIVE, .SERVLET
                .bannerMode(Banner.Mode.OFF)
                .run(args);
    }

    Picture[] pictures = new Picture[2];
    int flag = 0;
    @Override
    public void run(ApplicationArguments args) {
        ServerSocket serverSocket = null;
        InputStream inputStream = null;
        ObjectInputStream objectInputStream = null;
        try {
            serverSocket = new ServerSocket(2020);
            while (true) {
                //接收数据包
                Socket client = serverSocket.accept();
                inputStream = client.getInputStream();
                objectInputStream = new ObjectInputStream(new BufferedInputStream(inputStream));
                Object object = objectInputStream.readObject();
                //收到数据包
                DataPackage dataPackage = (DataPackage) object;
                System.out.println("receive");
                //构建新的数据包
                DataPackage newDataPackage = dataPackage;
                //得到本机路由表
                String[] route = findService.findRoute(dataPackage.getCurrentSataHop(), dataPackage.getRoute());
                System.out.println(Arrays.toString(route));
                System.out.println("========================");
                //判断是否为该边的终点,如果是，进行函数计算并创建一个新的数据包,转发
                if (dataPackage.getCurrentSataHop().equals(dataPackage.getDestinSataHop())) {
                    LinkedList<Picture> picPara = new LinkedList<>();
                    //此时为终点
                    String task = findService.findTask(dataPackage.getCurrentSataHop(), dataPackage.getTaskTable());
                    switch (task) {
                        case "binaryProcess":
                            picPara = new binaryProcess().picProcess(dataPackage.getPicture());
                            break;
                        case "cutProcess":
                            picPara = new cutProcess().picProcess(dataPackage.getPicture());
                            break;
                        case "greyProcess":
                            picPara = new greyProcess().picProcess(dataPackage.getPicture());
                            break;
                        case "mergeProcess":
                            if (flag==0){
                                pictures[flag] = dataPackage.getPicture();
                                System.out.println("pictures1:"+pictures.length);
                                flag =1;
                                continue;
                            }else {
                                pictures[flag] = dataPackage.getPicture();
                                System.out.println("pictures2:"+pictures.length);
                                flag =0;
                            }
                            System.out.println("merge Operate");
                            System.out.println(pictures.length);
                            picPara = new mergeProcess().picProcess(pictures);
                            break;
                        case "tailorProcess":
                            picPara = new tailorProcess().picProcess(dataPackage.getPicture());
                            break;
                    }
//                    System.out.println("res = "+ picProcess.getRes());
                    //res不等于-1，说明计算过程正常
//                    if (picProcess.getRes() != -1) {
/*                    //写入更新后的数据
                    newDataPackage.setData(picProcess.getRes());*/

                    //找到目的地卫星数组
                    ArrayList<String> destinSataHops = findService.findDestinSataHops(dataPackage.getPathTable(), dataPackage.getCurrentSataHop());
                    //最终卫星节点，不做任务处理，仅接受数据
                    if(destinSataHops.get(0).equals("nolast")){
                         System.out.println("pathcomputing end");
                         //todo:输出到服务器指定文件夹
                        continue;
                    }
                    for (String destinSataHop : destinSataHops) {
                        //设置目的地址
                        newDataPackage.setDestinSataHop(destinSataHop);
                        //设置下一跳地址
                        newDataPackage.setNextSataHop(findService.findNextHop(destinSataHop, route));
                        //设置本机地址
                        newDataPackage.setCurrentSataHop(dataPackage.getNextSataHop());
                        //创建时间
                        Date date = new Date();
                        long time = date.getTime();
                        //转发
                        newDataPackage.setPicture(picPara.removeFirst());
                        transferService.transferPackage(newDataPackage, newDataPackage.getNextSataHop(), time);
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
//                    }
                } else {//仅作转发的节点
                    System.out.println("only transfer nextHop"+findService.findNextHop(dataPackage.getDestinSataHop(), route));
                    newDataPackage.setNextSataHop(findService.findNextHop(dataPackage.getDestinSataHop(), route));
                    System.out.println("only current nextHop"+dataPackage.getNextSataHop());
                    newDataPackage.setCurrentSataHop(dataPackage.getNextSataHop());
                    //创建时间
                    Date date = new Date();
                    long time = date.getTime();
                    //转发
                    transferService.transferPackage(newDataPackage, newDataPackage.getNextSataHop(), time);
                }
            }
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace(); } finally {
            try {
                if (objectInputStream != null) {
                    objectInputStream.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
                if (serverSocket != null) {
                    serverSocket.close();
                }
            } catch (IOException e) {
                e.printStackTrace(); }
        }
    }
}






















/*
@Service
@SpringBootApplication
public class Application implements ApplicationRunner {

    @Autowired
    private ServerService serverService;
    @Autowired
    private IPService ipService;
    @Value("${slot}")
    private Long slot;  //时隙值

    public static void main(String[] args) {
        new SpringApplicationBuilder(Application.class)
                .web(WebApplicationType.NONE) // .REACTIVE, .SERVLET
                .bannerMode(Banner.Mode.OFF)
                .run(args);
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        ServerSocket serverSocket = null;
        InputStream inputStream = null;
        ObjectInputStream objectInputStream = null;
        Long startime = System.currentTimeMillis();
        try {
            serverSocket = new ServerSocket(2020);
            while (true) {
                Socket client = serverSocket.accept();
                inputStream = client.getInputStream();
                objectInputStream = new ObjectInputStream(new BufferedInputStream(inputStream));
                Object object = objectInputStream.readObject();
                DataPackage dataPackage = (DataPackage) object;

                Stack[] route = dataPackage.getRoute();
                int routeNumber = dataPackage.getRouteNumber();
                int[] mapping = {0,1,2,3,4,5,5,6};
                List<Integer> sequencet = dataPackage.getSequence();
                int currentHop = dataPackage.getCurrentHop();
                if (!route[routeNumber].empty()){
                    //发送到其他卫星计算，本卫星只转发
                    //get下一转发节点
                    Integer nextHop = (Integer) route[routeNumber].pop();
                    System.out.println("cover transfer next sate:"+nextHop);
                    //重新组合数据包
                    dataPackage.setRoute(route);
                    dataPackage.setCurrentHop(nextHop);
                    transferPackage(dataPackage, nextHop, startime);
                    continue;
                }else if(sequencet.get(mapping[routeNumber])!=currentHop){

                    int nextHop = sequencet.get(mapping[routeNumber]);
                    System.out.println("routeNumber"+routeNumber);
                    System.out.println("currentHop:"+currentHop+"  task exec sate"+nextHop);
                    dataPackage.setCurrentHop(nextHop);
                    transferPackage(dataPackage, nextHop, startime);
                    continue;
                }


                int number = dataPackage.getNumber();
                List<Integer> sequence = dataPackage.getSequence();
                Picture picture = dataPackage.getPicture();
                String loc = dataPackage.getLoc();
                double eta = dataPackage.getEta();

                Long Currentime = dataPackage.getCurrentime();
                Long fun1time = dataPackage.getFun1time();
                List<Double> compCapacity = dataPackage.getCompCapacity();
                LinkedList<Double> tranDelay =dataPackage.getTranDelay();
                System.out.println("route array"+tranDelay);
                System.out.println("=====================");
                System.out.println("running");

                String methodName = "func" + number;
                Method method = serverService.getClass().getMethod(methodName, Class.forName("java.util.List"), Picture.class, Class.forName("java.lang.String"), Class.forName("java.lang.Double"),
                        Class.forName("java.lang.Long"),Class.forName("java.lang.Long"), Class.forName("java.lang.Long"), Class.forName("java.util.List"), route.getClass(), Class.forName("java.util.LinkedList"));
                method.invoke(serverService, sequence, picture, loc, eta, Currentime, startime, fun1time, compCapacity, route, tranDelay);
            }
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
        } finally {
            try {
                if (objectInputStream != null) {
                    objectInputStream.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
                if (serverSocket != null) {
                    serverSocket.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void transferPackage(DataPackage dataPackage, int nextHop, long startime) {
        //获取目标节点ip
        String app_name = "sate" + nextHop;
        String IP = ipService.getIpByAppName(app_name);
        //发送至目标
        send(dataPackage, IP, 2020, startime);
    }
    private void send(DataPackage dataPackage, String ip, int port, Long startime) {
        Socket socket = null;
        try {
            socket = new Socket(ip, port);
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(socket.getOutputStream());
            objectOutputStream.writeObject(dataPackage);
        } catch (IOException e) {
            Long waitime = slot-(System.currentTimeMillis()-startime)%slot;
            try {
                Thread.sleep(waitime+10);
            } catch (InterruptedException ex) {
                ex.printStackTrace();
            }
            System.out.println(">>>>>>>>>>>>>>>>>>>");
            System.out.println("blocked current slot: " + ((System.currentTimeMillis()-startime)/slot + 1));
            System.out.println("try again ...");
            send(dataPackage, ip, port, startime);
        } finally {
            try {
                if (socket != null) {
                    socket.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
*/
